@@ -12,7 +12,7 @@
 #include <rime/engine.h>
 #include <rime/dict/vocabulary.h>
 #include <rime/gear/charset_filter.h>
-#include <boost/locale/encoding.hpp>
+
 
 namespace rime {
 
@@ -24,6 +24,8 @@ bool is_extended_cjk(uint32_t ch)
       (ch >= 0x2B740 && ch <= 0x2B81F) ||  // CJK Unified Ideographs Extension D
       (ch >= 0x2B820 && ch <= 0x2CEAF) ||  // CJK Unified Ideographs Extension E
       (ch >= 0x2CEB0 && ch <= 0x2EBEF) ||  // CJK Unified Ideographs Extension F
+      (ch >= 0x30000 && ch <= 0x3134F) ||  // CJK Unified Ideographs Extension G
+      (ch >= 0xF900 && ch <= 0xFAFF) ||    // CJK Compatibility Ideographs
       (ch >= 0x2F800 && ch <= 0x2FA1F))    // CJK Compatibility Ideographs Supplement
     return true;
 
@@ -46,9 +48,8 @@ bool contains_extended_cjk(const string& text)
 
 // CharsetFilterTranslation
 
-CharsetFilterTranslation::CharsetFilterTranslation(
-    an<Translation> translation, const string& charset)
-    : translation_(translation), charset_(charset) {
+CharsetFilterTranslation::CharsetFilterTranslation(an<Translation> translation)
+    : translation_(translation) {
   LocateNextCandidate();
 }
 
@@ -66,10 +67,14 @@ an<Candidate> CharsetFilterTranslation::Peek() {
   return translation_->Peek();
 }
 
+bool CharsetFilterTranslation::FilterCandidate(an<Candidate> cand) {
+  return CharsetFilter::FilterText(cand->text());
+}
+
 bool CharsetFilterTranslation::LocateNextCandidate() {
   while (!translation_->exhausted()) {
     auto cand = translation_->Peek();
-    if (cand && CharsetFilter::FilterText(cand->text(), charset_))
+    if (cand && FilterCandidate(cand))
       return true;
     translation_->Next();
   }
@@ -79,17 +84,8 @@ bool CharsetFilterTranslation::LocateNextCandidate() {
 
 // CharsetFilter
 
-bool CharsetFilter::FilterText(const string& text, const string& charset) {
-  if (charset.empty()) return !contains_extended_cjk(text);
-  try {
-    boost::locale::conv::from_utf(text, charset, boost::locale::conv::method_type::stop);
-  }
-  catch(boost::locale::conv::conversion_error const& /*ex*/) {
-    return false;
-  }
-  catch(...) {
-  }
-  return true;
+bool CharsetFilter::FilterText(const string& text) {
+  return !contains_extended_cjk(text);
 }
 
 bool CharsetFilter::FilterDictEntry(an<DictEntry> entry) {
@@ -102,11 +98,12 @@ CharsetFilter::CharsetFilter(const Ticket& ticket)
 
 an<Translation> CharsetFilter::Apply(
     an<Translation> translation, CandidateList* candidates) {
-  if (name_space_.empty() && !engine_->context()->get_option("extended_charset")) {
+  if (name_space_.empty() &&
+      !engine_->context()->get_option("extended_charset")) {
     return New<CharsetFilterTranslation>(translation);
   }
-  if (!name_space_.empty() && engine_->context()->get_option(name_space_)) {
-    return New<CharsetFilterTranslation>(translation, name_space_);
+  if (!name_space_.empty()) {
+    LOG(ERROR) << "charset parameter is unsupported by basic charset_filter";
   }
   return translation;
 }
